@@ -16,7 +16,6 @@ module Happstack.Server.Internal.Cookie
     )
     where
 
-import Control.Applicative   ((<$>))
 import qualified Data.ByteString.Char8 as C
 import Data.Char             (chr, toLower)
 import Data.Data             (Data, Typeable)
@@ -24,7 +23,6 @@ import Data.List             ((\\), intersperse)
 import Data.Time.Clock       (UTCTime, addUTCTime, diffUTCTime)
 import Data.Time.Clock.POSIX (posixSecondsToUTCTime)
 import Data.Time.Format      (formatTime)
-import Happstack.Util.Common (Seconds)
 import Happstack.Server.Internal.Clock (getApproximateUTCTime)
 import Text.ParserCombinators.Parsec hiding (token)
 import System.Locale         (defaultTimeLocale)
@@ -37,6 +35,7 @@ data Cookie = Cookie
     , cookieName    :: String
     , cookieValue   :: String
     , secure        :: Bool
+    , httpOnly      :: Bool
     } deriving(Show,Eq,Read,Typeable,Data)
 
 -- | Specify the lifetime of a cookie.
@@ -48,13 +47,13 @@ data Cookie = Cookie
 --
 data CookieLife
     = Session         -- ^ session cookie - expires when browser is closed
-    | MaxAge Seconds  -- ^ life time of cookie in seconds
+    | MaxAge Int      -- ^ life time of cookie in seconds
     | Expires UTCTime -- ^ cookie expiration date
     | Expired         -- ^ cookie already expired
       deriving (Eq, Ord, Read, Show, Typeable)
 
 -- convert 'CookieLife' to the argument needed for calling 'mkCookieHeader'
-calcLife :: CookieLife -> IO (Maybe (Seconds, UTCTime))
+calcLife :: CookieLife -> IO (Maybe (Int, UTCTime))
 calcLife Session = return Nothing
 calcLife (MaxAge s) =
           do now <- getApproximateUTCTime
@@ -66,13 +65,14 @@ calcLife Expired =
           return $ Just (0, posixSecondsToUTCTime 0)
 
 
--- | Creates a cookie with a default version of 1, empty domain, a path of "/", and secure == False
+-- | Creates a cookie with a default version of 1, empty domain, a
+-- path of "/", secure == False and httpOnly == False
 --
 -- see also: 'addCookie'
 mkCookie :: String  -- ^ cookie name
          -> String  -- ^ cookie value
          -> Cookie
-mkCookie key val = Cookie "1" "/" "" key val False
+mkCookie key val = Cookie "1" "/" "" key val False False
 
 -- | Set a Cookie in the Result.
 -- The values are escaped as per RFC 2109, but some browsers may
@@ -87,7 +87,7 @@ mkCookie key val = Cookie "1" "/" "" key val False
 --
 -- See 'CookieLife' and 'calcLife' for a convenient way of calculating
 -- the first argument to this function.
-mkCookieHeader :: Maybe (Seconds, UTCTime) -> Cookie -> String
+mkCookieHeader :: Maybe (Int, UTCTime) -> Cookie -> String
 mkCookieHeader mLife cookie =
     let l = [("Domain=",  cookieDomain cookie)
             ,("Max-Age=", maybe "" (show . max 0 . fst) mLife)
@@ -98,7 +98,9 @@ mkCookieHeader mLife cookie =
         s f   = '\"' : concatMap e (f cookie) ++ "\""
         e c | fctl c || c == '"' = ['\\',c]
             | otherwise          = [c]
-    in concat $ intersperse ";" ((cookieName cookie++"="++s cookieValue):[ (k++v) | (k,v) <- l, "" /= v ] ++ if secure cookie then ["Secure"] else [])
+    in concat $ intersperse ";" ((cookieName cookie++"="++s cookieValue):[ (k++v) | (k,v) <- l, "" /= v ] ++ 
+                                 (if secure cookie then ["Secure"] else []) ++
+                                 (if httpOnly cookie then ["HttpOnly"] else []))
 
 fctl :: Char -> Bool
 fctl ch = ch == chr 127 || ch <= chr 31
@@ -125,7 +127,7 @@ cookiesParser = cookies
             val<-value
             path<-option "" $ try (cookieSep >> cookie_path)
             domain<-option "" $ try (cookieSep >> cookie_domain)
-            return $ Cookie ver path domain (low name) val False
+            return $ Cookie ver path domain (low name) val False False
           cookie_version = cookie_special "$Version"
           cookie_path = cookie_special "$Path"
           cookie_domain = cookie_special "$Domain"
@@ -137,12 +139,12 @@ cookiesParser = cookies
           cookieEq = ws >> char '=' >> ws
           ws = spaces
           value         = word
-          word          = try (quoted_string) <|> incomp_token
+          word          = try quoted_string <|> try incomp_token <|> return ""
 
           -- Parsers based on RFC 2068
           quoted_string = do
             char '"'
-            r <-many (oneOf qdtext)
+            r <-many ((try quotedPair) <|> (oneOf qdtext))
             char '"'
             return r
 
@@ -156,6 +158,7 @@ cookiesParser = cookies
           octet         = map chr [0..255]
           text          = octet \\ ctl
           qdtext        = text \\ "\""
+          quotedPair    = char '\\' >> anyChar
 
 -- | Get all cookies from the HTTP request. The cookies are ordered per RFC from
 -- the most specific to the least specific. Multiple cookies with the same
