@@ -14,7 +14,7 @@ module Happstack.Server.Monads
       ServerPartT
     , ServerPart
       -- * Happstack class
-    , Happstack(..)
+    , Happstack
       -- * ServerMonad
     , ServerMonad(..)
     , mapServerPartT
@@ -36,19 +36,30 @@ module Happstack.Server.Monads
     , requireM
     ) where
 
+import Control.Applicative               (Alternative, Applicative)         
 import Control.Monad                     (MonadPlus(mzero))
 import Control.Monad.Trans               (MonadIO(..),MonadTrans(lift))
+import Control.Monad.Reader              (ReaderT)
+import Control.Monad.Writer              (WriterT)
+import Control.Monad.State               (StateT)
+import Control.Monad.RWS                 (RWST)
 import qualified Data.ByteString.Char8   as B
+import Data.Monoid                       (Monoid)
 import Happstack.Server.Internal.Monads
 import Happstack.Server.Types            (Response, addHeader, getHeader, setHeader)
 import Happstack.Server.RqData           (HasRqData)
 
 -- | A class alias for all the classes a standard server monad (such as 'ServerPartT') is expected to have instances for. This allows you to keep your type signatures shorter and easier to understand.
 class ( ServerMonad m, WebMonad Response m, FilterMonad Response m
-      , MonadIO m, MonadPlus m, HasRqData m, Monad m, Functor m) => Happstack m
+      , MonadIO m, MonadPlus m, HasRqData m, Monad m, Functor m
+      , Applicative m, Alternative m) => Happstack m
 
-
-instance (Functor m, Monad m, MonadPlus m, MonadIO m) => Happstack (ServerPartT m)
+instance (Functor m, Monad m, MonadPlus m
+         , MonadIO m)            => Happstack (ServerPartT m)
+instance (Happstack m)           => Happstack (StateT      s m)
+instance (Happstack m)           => Happstack (ReaderT r     m)
+instance (Happstack m, Monoid w) => Happstack (WriterT   w   m)
+instance (Happstack m, Monoid w) => Happstack (RWST    r w s m)
 
 -- | Get a header out of the request.
 getHeaderM :: (ServerMonad m) => String -> m (Maybe B.ByteString)
