@@ -1,4 +1,4 @@
-{-# LANGUAGE DeriveDataTypeable, GeneralizedNewtypeDeriving, FlexibleInstances, MultiParamTypeClasses #-}
+{-# LANGUAGE DeriveDataTypeable, GeneralizedNewtypeDeriving, FlexibleContexts, FlexibleInstances, MultiParamTypeClasses #-}
 -- | Functions for extracting values from the query string, form data, cookies, etc. 
 --
 -- For in-depth documentation see the following section of the Happstack Crash Course:
@@ -10,7 +10,9 @@ module Happstack.Server.RqData
       look
     , looks
     , lookText
+    , lookText'
     , lookTexts
+    , lookTexts'
     , lookBS
     , lookBSs
     , lookRead
@@ -30,10 +32,12 @@ module Happstack.Server.RqData
     -- body for matches keys. 
     , body
     , queryString
+    , bytestring
     -- * Validation and Parsing
     , checkRq
     , checkRqM        
     , readRq
+    , unsafeReadRq
     -- * Handling POST\/PUT Requests
     , decodeBody
     -- ** Body Policy
@@ -56,10 +60,14 @@ module Happstack.Server.RqData
 
 import Control.Applicative 			(Applicative((<*>), pure), Alternative((<|>), empty), WrappedMonad(WrapMonad, unwrapMonad), (<$>))
 import Control.Concurrent.MVar                  (newMVar)
-import Control.Monad 				(MonadPlus(mzero), liftM)
+import Control.Monad 				(MonadPlus(mzero))
 import Control.Monad.Reader 			(ReaderT(ReaderT, runReaderT), MonadReader(ask, local), mapReaderT)
+import Control.Monad.State 			(StateT, mapStateT)
+import Control.Monad.Writer 			(WriterT, mapWriterT)
+import Control.Monad.RWS  			(RWST, mapRWST)
 import Control.Monad.Error 			(Error(noMsg, strMsg))
-import Control.Monad.Trans                      (MonadIO(..))
+import Control.Monad.Trans                      (MonadIO(..), lift)
+import qualified Data.ByteString.Char8          as P
 import qualified Data.ByteString.Lazy.Char8     as L
 import qualified Data.ByteString.Lazy.UTF8      as LU
 import Data.Char 				(toLower)
@@ -67,12 +75,15 @@ import Data.Either                              (partitionEithers)
 import Data.Generics                            (Data, Typeable)
 import Data.Maybe                               (fromMaybe, fromJust)
 import Data.Monoid 				(Monoid(mempty, mappend, mconcat))
-import           Data.Text.Lazy                 (Text)
-import qualified Data.Text.Lazy.Encoding        as Text
+import           Data.Text                      (Text)
+import qualified Data.Text.Lazy                 as LazyText
+import qualified Data.Text.Lazy.Encoding        as LazyText
 import Happstack.Server.Cookie 			(Cookie (cookieValue))
-import Happstack.Server.Internal.Monads         (ServerMonad(askRq, localRq), ServerPartT)
-import Happstack.Server.Types                   (ContentType(..), Input(inputValue, inputFilename, inputContentType), Request(rqInputsQuery, rqInputsBody, rqCookies, rqMethod), Method(POST,PUT), readInputsBody)
+import Happstack.Server.Internal.Monads         (ServerMonad(askRq, localRq), FilterMonad, WebMonad, ServerPartT, escape)
+import Happstack.Server.Internal.RFC822Headers  (parseContentType)
+import Happstack.Server.Types                   (ContentType(..), FromReqURI(..), Input(inputValue, inputFilename, inputContentType), Response, Request(rqInputsQuery, rqInputsBody, rqCookies, rqMethod), Method(POST,PUT), getHeader, readInputsBody)
 import Happstack.Server.Internal.MessageWrap    (BodyPolicy(..), bodyInput, defaultBodyPolicy)
+import Happstack.Server.Response                (requestEntityTooLarge, toResponse)
 
 newtype ReaderError r e a = ReaderError { unReaderError :: ReaderT r (Either e) a }
     deriving (Functor, Monad, MonadPlus)
@@ -109,18 +120,20 @@ instance Error (Errors String) where
     noMsg = Errors []
     strMsg str = Errors [str]
 
-mapReaderErrorT :: (Either e a -> Either e' b) -> (ReaderError r e a) -> (ReaderError r e' b)
-mapReaderErrorT f m = ReaderError $ mapReaderT f (unReaderError m)
-
+{- commented out to avoid 'Defined but not used' warning. 
 readerError :: (Monoid e, Error e) => e -> ReaderError r e b
 readerError e = mapReaderErrorT ((Left e) `apEither`) (return ())
 
+mapReaderErrorT :: (Either e a -> Either e' b) -> (ReaderError r e a) -> (ReaderError r e' b)
+mapReaderErrorT f m = ReaderError $ mapReaderT f (unReaderError m)
+-}
+
 runReaderError :: ReaderError r e a -> r -> Either e a
 runReaderError = runReaderT . unReaderError
 
 -- | the environment used to lookup query parameters. It consists of
 -- the triple: (query string inputs, body inputs, cookie inputs)
-type RqEnv = ([(String, Input)], [(String, Input)], [(String, Cookie)])
+type RqEnv = ([(String, Input)], Maybe [(String, Input)], [(String, Cookie)])
 
 -- | An applicative functor and monad for looking up key/value pairs
 -- in the QUERY_STRING, Request body, and cookies.
@@ -142,25 +155,56 @@ instance HasRqData RqData where
 -- instance (MonadPlus m, MonadIO m, ServerMonad m) => (HasRqData m) where
 instance (MonadIO m) => HasRqData (ServerPartT m) where
     askRqEnv =
-        do rq <- askRq
-           mbi <- liftIO $ if ((rqMethod rq) == POST) || ((rqMethod rq) == PUT)
+        do rq  <- askRq
+           mbi <- liftIO $ if ((rqMethod rq == POST) || (rqMethod rq == PUT)) && (isDecodable (ctype rq))
                            then readInputsBody rq
                            else return (Just [])
-           case mbi of
-             Nothing   -> fail "askRqEnv failed because the request body has not been decoded yet. Try using 'decodeBody'."
-             (Just bi) -> return (rqInputsQuery rq, bi, rqCookies rq)
-    rqDataError e = mzero
+           return (rqInputsQuery rq, mbi, rqCookies rq)
+        where
+          ctype :: Request -> Maybe ContentType
+          ctype req = parseContentType . P.unpack =<< getHeader "content-type" req
+          isDecodable :: Maybe ContentType -> Bool
+          isDecodable Nothing                                                      = True -- assume it is application/x-www-form-urlencoded
+          isDecodable (Just (ContentType "application" "x-www-form-urlencoded" _)) = True
+          isDecodable (Just (ContentType "multipart" "form-data" _ps))             = True
+          isDecodable (Just _)                                                     = False
+
+    rqDataError _e = mzero
     localRqEnv f m =
         do rq <- askRq
-           b  <- liftM (fromMaybe []) $ liftIO $ readInputsBody rq
+           b  <- liftIO $ readInputsBody rq
            let (q', b', c') = f (rqInputsQuery rq, b, rqCookies rq)
-           bv <- liftIO $ newMVar b'
+           bv <- liftIO $ newMVar (fromMaybe [] b')
            let rq' = rq { rqInputsQuery = q'
                         , rqInputsBody = bv
                         , rqCookies = c'
                         }
            localRq (const rq') m
 
+------------------------------------------------------------------------------
+-- HasRqData instances for ReaderT, StateT, WriterT, and RWST
+------------------------------------------------------------------------------
+
+instance (Monad m, HasRqData m) => HasRqData (ReaderT s m) where
+    askRqEnv      = lift askRqEnv
+    localRqEnv f  = mapReaderT (localRqEnv f)
+    rqDataError e = lift (rqDataError e)
+
+instance (Monad m, HasRqData m) => HasRqData (StateT s m) where
+    askRqEnv      = lift askRqEnv
+    localRqEnv f  = mapStateT (localRqEnv f)
+    rqDataError e = lift (rqDataError e)
+
+instance (Monad m, HasRqData m, Monoid w) => HasRqData (WriterT w m) where
+    askRqEnv      = lift askRqEnv
+    localRqEnv f  = mapWriterT (localRqEnv f)
+    rqDataError e = lift (rqDataError e)
+
+instance (Monad m, HasRqData m, Monoid w) => HasRqData (RWST r w s m) where
+    askRqEnv      = lift askRqEnv
+    localRqEnv f  = mapRWST (localRqEnv f)
+    rqDataError e = lift (rqDataError e)
+
 -- | apply 'RqData a' to a 'RqEnv'
 --
 -- see also: 'getData', 'getDataFn', 'withData', 'withDataFn', 'RqData', 'getDataFn'
@@ -177,18 +221,41 @@ mapRqData f m = RqData $ ReaderError $ mapReaderT f (unReaderError (unRqData m))
 
 -- | use 'read' to convert a 'String' to a value of type 'a'
 --
+-- > look "key" `checkRq` (unsafeReadRq "key")
+-- 
+-- use with 'checkRq'
+--
+-- NOTE: This function is marked unsafe because some Read instances
+-- are vulnerable to attacks that attempt to create an out of memory
+-- condition. For example:
+--
+-- > read "1e10000000000000" :: Integer
+--
+-- see also: 'readRq'
+unsafeReadRq :: (Read a) => 
+          String -- ^ name of key (only used for error reporting)
+       -> String -- ^ 'String' to 'read'
+       -> Either String a -- ^ 'Left' on error, 'Right' on success
+unsafeReadRq key val =
+    case reads val of
+      [(a,[])] -> Right a
+      _        -> Left $ "readRq failed while parsing key: " ++ key ++ " which has the value: " ++ val
+      
+-- | use 'fromReqURI' to convert a 'String' to a value of type 'a'
+--
 -- > look "key" `checkRq` (readRq "key")
 -- 
 -- use with 'checkRq'
-readRq :: (Read a) => 
+readRq :: (FromReqURI a) => 
           String -- ^ name of key (only used for error reporting)
        -> String -- ^ 'String' to 'read'
        -> Either String a -- ^ 'Left' on error, 'Right' on success
 readRq key val =
-    case reads val of
-      [(a,[])] -> Right a
+    case fromReqURI val of
+      (Just a) -> Right a
       _        -> Left $ "readRq failed while parsing key: " ++ key ++ " which has the value: " ++ val
 
+
 -- | convert or validate a value
 --
 -- This is similar to 'fmap' except that the function can fail by
@@ -214,9 +281,9 @@ checkRq rq f =
 -- | like 'checkRq' but the check function can be monadic
 checkRqM :: (Monad m, HasRqData m) => m a -> (a -> m (Either String b)) -> m b
 checkRqM rq f =
-    do a <- rq
-       b <- f a
-       case b of
+    do a  <- rq
+       eb <- f a
+       case eb of
          (Left e)  -> rqDataError (strMsg e)
          (Right b) -> return b
 
@@ -248,6 +315,12 @@ instance FromData a => FromData (Maybe a) where
 lookups :: (Eq a) => a -> [(a, b)] -> [b]
 lookups a = map snd . filter ((a ==) . fst)
 
+fromMaybeBody :: String -> String -> Maybe [(String, Input)] -> [(String, Input)]
+fromMaybeBody funName fieldName mBody =
+    case mBody of
+      Nothing -> error $ funName ++ " " ++ fieldName ++ " failed because the request body has not been decoded yet. Try using 'decodeBody' to decode the body. Or the 'queryString' filter to ignore the body."
+      (Just bdy) -> bdy
+
 -- | Gets the first matching named input parameter
 -- 
 -- Searches the QUERY_STRING followed by the Request body.
@@ -255,8 +328,9 @@ lookups a = map snd . filter ((a ==) . fst)
 -- see also: 'lookInputs'
 lookInput :: (Monad m, HasRqData m) => String -> m Input
 lookInput name
-    = do (query, body, _cookies) <- askRqEnv
-         case lookup name (query ++ body) of
+    = do (query, mBody, _cookies) <- askRqEnv
+         let bdy = fromMaybeBody "lookInput" name mBody
+         case lookup name (query ++ bdy) of
            Just i  -> return $ i
            Nothing -> rqDataError (strMsg $ "Parameter not found: " ++ name)
 
@@ -267,8 +341,9 @@ lookInput name
 -- see also: 'lookInput'
 lookInputs :: (Monad m, HasRqData m) => String -> m [Input]
 lookInputs name
-    = do (query, body, _cookies) <- askRqEnv
-         return $ lookups name (query ++ body)
+    = do (query, mBody, _cookies) <- askRqEnv
+         let bdy = fromMaybeBody "lookInputs" name mBody
+         return $ lookups name (query ++ bdy)
 
 -- | Gets the first matching named input parameter as a lazy 'ByteString'
 --
@@ -279,7 +354,7 @@ lookBS :: (Functor m, Monad m, HasRqData m) => String -> m L.ByteString
 lookBS n = 
     do i <- fmap inputValue (lookInput n)
        case i of
-         (Left fp)  -> rqDataError $ (strMsg $ "lookBS: " ++ n ++ " is a file.")
+         (Left _fp) -> rqDataError $ (strMsg $ "lookBS: " ++ n ++ " is a file.")
          (Right bs) -> return bs
 
 -- | Gets all matches for the named input parameter as lazy 'ByteString's
@@ -292,7 +367,7 @@ lookBSs n =
     do is <- fmap (map inputValue) (lookInputs n)
        case partitionEithers is of
          ([], bs) -> return bs
-         (fp, _)  -> rqDataError (strMsg $ "lookBSs: " ++ n ++ " is a file.")
+         (_fp, _) -> rqDataError (strMsg $ "lookBSs: " ++ n ++ " is a file.")
 
 -- | Gets the first matching named input parameter as a 'String'
 --
@@ -328,8 +403,18 @@ looks = fmap (map LU.toString) . lookBSs
 -- This function assumes the underlying octets are UTF-8 encoded.
 --
 -- see also: 'lookTexts', 'look', 'looks', 'lookBS', and 'lookBSs'
-lookText :: (Functor m, Monad m, HasRqData m) => String -> m Text
-lookText = fmap Text.decodeUtf8 . lookBS
+lookText :: (Functor m, Monad m, HasRqData m) => String -> m LazyText.Text
+lookText = fmap LazyText.decodeUtf8 . lookBS
+
+-- | Gets the first matching named input parameter as a strict 'Text'
+--
+-- Searches the QUERY_STRING followed by the Request body.
+--
+-- This function assumes the underlying octets are UTF-8 encoded.
+--
+-- see also: 'lookTexts', 'look', 'looks', 'lookBS', and 'lookBSs'
+lookText' :: (Functor m, Monad m, HasRqData m) => String -> m Text
+lookText' = fmap LazyText.toStrict . lookText
 
 -- | Gets all matches for the named input parameter as lazy 'Text's
 --
@@ -338,8 +423,18 @@ lookText = fmap Text.decodeUtf8 . lookBS
 -- This function assumes the underlying octets are UTF-8 encoded.
 --
 -- see also: 'lookText', 'looks' and 'lookBSs'
-lookTexts :: (Functor m, Monad m, HasRqData m) => String -> m [Text]
-lookTexts = fmap (map Text.decodeUtf8) . lookBSs
+lookTexts :: (Functor m, Monad m, HasRqData m) => String -> m [LazyText.Text]
+lookTexts = fmap (map LazyText.decodeUtf8) . lookBSs
+
+-- | Gets all matches for the named input parameter as strict 'Text's
+--
+-- Searches the QUERY_STRING followed by the Request body.
+--
+-- This function assumes the underlying octets are UTF-8 encoded.
+--
+-- see also: 'lookText'', 'looks' and 'lookBSs'
+lookTexts' :: (Functor m, Monad m, HasRqData m) => String -> m [Text]
+lookTexts' = fmap (map LazyText.toStrict) . lookTexts
 
 -- | Gets the named cookie
 -- the cookie name is case insensitive
@@ -355,7 +450,7 @@ lookCookieValue :: (Functor m, Monad m, HasRqData m) => String -> m String
 lookCookieValue = fmap cookieValue . lookCookie
 
 -- | gets the named cookie as the requested Read type
-readCookieValue :: (Functor m, Monad m, HasRqData m, Read a) => String -> m a
+readCookieValue :: (Functor m, Monad m, HasRqData m, FromReqURI a) => String -> m a
 readCookieValue name = fmap cookieValue (lookCookie name) `checkRq` (readRq name)
 
 -- | Gets the first matching named input parameter and decodes it using 'Read'
@@ -365,7 +460,7 @@ readCookieValue name = fmap cookieValue (lookCookie name) `checkRq` (readRq name
 -- This function assumes the underlying octets are UTF-8 encoded.
 --
 -- see also: 'lookReads'
-lookRead :: (Functor m, Monad m, HasRqData m, Read a) => String -> m a
+lookRead :: (Functor m, Monad m, HasRqData m, FromReqURI a) => String -> m a
 lookRead name = look name `checkRq` (readRq name)
 
 -- | Gets all matches for the named input parameter and decodes them using 'Read'
@@ -375,7 +470,7 @@ lookRead name = look name `checkRq` (readRq name)
 -- This function assumes the underlying octets are UTF-8 encoded.
 --
 -- see also: 'lookReads'
-lookReads :: (Functor m, Monad m, HasRqData m, Read a) => String -> m [a]
+lookReads :: (Functor m, Monad m, HasRqData m, FromReqURI a) => String -> m [a]
 lookReads name = 
     do vals <- looks name
        mapM (\v -> (return v) `checkRq` (readRq name)) vals
@@ -415,8 +510,9 @@ lookFile n =
 -- see also: 'lookPairsBS'
 lookPairs :: (Monad m, HasRqData m) => m [(String, Either FilePath String)]
 lookPairs = 
-    do (query, body, _cookies) <- askRqEnv
-       return $ map (\(n,vbs)->(n, (\e -> case e of Left fp -> Left fp ; Right bs -> Right (LU.toString bs)) $ inputValue vbs)) (query ++ body)
+    do (query, mBody, _cookies) <- askRqEnv
+       let bdy = fromMaybeBody "lookPairs" "" mBody
+       return $ map (\(n,vbs)->(n, (\e -> case e of Left fp -> Left fp ; Right bs -> Right (LU.toString bs)) $ inputValue vbs)) (query ++ bdy)
 
 -- | gets all the input parameters
 --
@@ -426,8 +522,9 @@ lookPairs =
 -- see also: 'lookPairs'
 lookPairsBS :: (Monad m, HasRqData m) => m [(String, Either FilePath L.ByteString)]
 lookPairsBS = 
-    do (query, body, _cookies) <- askRqEnv
-       return $ map (\(n,vbs) -> (n, inputValue vbs)) (query ++ body)
+    do (query, mBody, _cookies) <- askRqEnv
+       let bdy = fromMaybeBody "lookPairsBS" "" mBody
+       return $ map (\(n,vbs) -> (n, inputValue vbs)) (query ++ bdy)
 
 -- | The POST\/PUT body of a Request is not received or decoded unless
 -- this function is invoked. 
@@ -447,13 +544,13 @@ lookPairsBS =
 -- Only the first call to 'decodeBody' will have any effect. Calling
 -- it a second time, even with different quota values, will do
 -- nothing.
-decodeBody :: (ServerMonad m, MonadPlus m, MonadIO m) => BodyPolicy -> m ()
+decodeBody :: (ServerMonad m, MonadPlus m, MonadIO m, FilterMonad Response m, WebMonad Response m) => BodyPolicy -> m ()
 decodeBody bp =
     do rq <- askRq
        (_, me) <- bodyInput bp rq
        case me of
          Nothing -> return ()
-         Just e  -> fail e -- FIXME: is this the best way to report the error
+         Just e  -> escape $ requestEntityTooLarge (toResponse e) -- FIXME: is this the best way to report the error
 
 -- | run 'RqData' in a 'ServerMonad'.
 --
@@ -482,7 +579,7 @@ decodeBody bp =
 --
 -- NOTE: you must call 'decodeBody' prior to calling this function if
 -- the request method is POST or PUT.
-getDataFn :: (HasRqData m, ServerMonad m, MonadIO m) => 
+getDataFn :: (HasRqData m, ServerMonad m) =>
              RqData a -- ^ 'RqData' monad to evaluate
           -> m (Either [String] a) -- ^ return 'Left' errors or 'Right' a
 getDataFn rqData =
@@ -494,7 +591,7 @@ getDataFn rqData =
 -- 
 -- NOTE: you must call 'decodeBody' prior to calling this function if
 -- the request method is POST or PUT.
-withDataFn :: (HasRqData m, MonadIO m, MonadPlus m, ServerMonad m) => RqData a -> (a -> m r) -> m r
+withDataFn :: (HasRqData m, MonadPlus m, ServerMonad m) => RqData a -> (a -> m r) -> m r
 withDataFn fn handle = getDataFn fn >>= either (const mzero) handle
 
 -- | A variant of 'getDataFn' that uses 'FromData' to chose your
@@ -524,14 +621,14 @@ withDataFn fn handle = getDataFn fn >>= either (const mzero) handle
 --
 -- NOTE: you must call 'decodeBody' prior to calling this function if
 -- the request method is POST or PUT.
-getData :: (HasRqData m, MonadIO m, ServerMonad m, FromData a) => m (Either [String] a)
+getData :: (HasRqData m, ServerMonad m, FromData a) => m (Either [String] a)
 getData = getDataFn fromData
 
 -- | similar to 'getData' except it calls a subhandler on success or 'mzero' on failure.
 --
 -- NOTE: you must call 'decodeBody' prior to calling this function if
 -- the request method is POST or PUT.
-withData :: (HasRqData m, MonadIO m, FromData a, MonadPlus m, ServerMonad m) => (a -> m r) -> m r
+withData :: (HasRqData m, FromData a, MonadPlus m, ServerMonad m) => (a -> m r) -> m r
 withData = withDataFn fromData
 
 -- | limit the scope to the Request body
@@ -543,7 +640,7 @@ withData = withDataFn fromData
 body :: (HasRqData m) => m a -> m a
 body rqData = localRqEnv f rqData
     where
-      f (_query, body, _cookies) = ([], body, [])
+      f (_query, bdy, _cookies) = ([], bdy, [])
 
 -- | limit the scope to the QUERY_STRING
 --
@@ -554,17 +651,14 @@ body rqData = localRqEnv f rqData
 queryString ::  (HasRqData m) => m a -> m a
 queryString rqData = localRqEnv f rqData
     where
-      f (query, _body, _cookies) = (query, [], [])
-
-right :: (MonadPlus m) => Either a b -> m b
-right (Right a) = return a
-right (Left e) = mzero
+      f (query, _body, _cookies) = (query, Just [], [])
 
+-- | limit the scope to 'Input's  which produce a 'ByteString' (aka, not a file)
 bytestring :: (HasRqData m) => m a -> m a
 bytestring rqData = localRqEnv f rqData
     where
-      f (query, body, cookies) = (filter bsf query, filter bsf body, cookies)
+      f (query, bdy, cookies) = (filter bsf query, filter bsf <$> bdy, cookies)
       bsf (_, i) =
           case inputValue i of
             (Left  _fp) -> False
-            (Right _bs) -> True
\ No newline at end of file
+            (Right _bs) -> True
