@@ -29,7 +29,7 @@ allTests :: Test
 allTests = 
     "happstack-server tests" ~: [ cookieParserTest
                                 , acceptEncodingParserTest
-                                , splitMultipart 
+                                , multipart
                                 , compressFilterResponseTest
                                 ]
 
@@ -38,21 +38,25 @@ cookieParserTest =
     "cookieParserTest" ~:
     [parseCookies "$Version=1;Cookie1=value1;$Path=\"/testpath\";$Domain=example.com;cookie2=value2"
         @?= (Right [
-            Cookie "1" "/testpath" "example.com" "cookie1" "value1" False
-          , Cookie "1" "" "" "cookie2" "value2" False
+            Cookie "1" "/testpath" "example.com" "cookie1" "value1" False False
+          , Cookie "1" "" "" "cookie2" "value2" False False
           ])
     ,parseCookies "  \t $Version = \"1\" ; cookie1 = \"randomcrap!@#%^&*()-_+={}[]:;'<>,.?/\\|\" , $Path=/  "
         @?= (Right [
-            Cookie "1" "/" "" "cookie1" "randomcrap!@#%^&*()-_+={}[]:;'<>,.?/\\|" False
+            Cookie "1" "/" "" "cookie1" "randomcrap!@#%^&*()-_+={}[]:;'<>,.?/|" False False
           ])
     ,parseCookies " cookie1 = value1  "
         @?= (Right [
-            Cookie "" "" "" "cookie1" "value1" False
+            Cookie "" "" "" "cookie1" "value1" False False
           ])
     ,parseCookies " $Version=\"1\";buggygooglecookie = valuewith=whereitshouldnotbe  "
         @?= (Right [
-            Cookie "1" "" "" "buggygooglecookie" "valuewith=whereitshouldnotbe" False
+            Cookie "1" "" "" "buggygooglecookie" "valuewith=whereitshouldnotbe" False False
           ])
+    , parseCookies "foo=\"\\\"bar\\\"\""
+        @?= (Right [
+              Cookie "" "" ""  "foo" "\"bar\"" False False
+             ])
     ]
 
 acceptEncodingParserTest :: Test
@@ -70,27 +74,37 @@ acceptEncodingParserTest =
        , (" x-gzip",[("x-gzip", Nothing)])
        ]
 
-splitMultipart :: Test
-splitMultipart =
-    "split multipart" ~: assertFailure "update splitMultipart test"
-{-
-    [ Just [pack "1"] @=? 
-           splitParts (pack "boundary")
-                      (pack "beg\r\n--boundary\r\n1\r\n--boundary--\r\nend")
-    , Just [pack "1\n"] @=? 
-           splitParts (pack "boundary")
-                      (pack "beg\r\n--boundary\r\n1\n\r\n--boundary--\r\nend")
-    , Just [pack "1\r\n"] @=? 
-           splitParts (pack "boundary")
-                      (pack "beg\r\n--boundary\r\n1\r\n\r\n--boundary--\r\nend")
-    , Just [pack "1\n\r"] @=? 
-           splitParts (pack "boundary")
-                      (pack "beg\r\n--boundary\r\n1\n\r\r\n--boundary--\r\nend")
-    , Just [pack "\r\n1\n\r"] @=? 
-           splitParts (pack "boundary")
-                      (pack "beg\r\n--boundary\r\n\r\n1\n\r\r\n--boundary--\r\nend")
+multipart :: Test
+multipart =
+    "split multipart" ~:
+    [ ([BodyPart (pack "content-type: text/plain\r\n") (pack "1")], Nothing) @=?
+           parseMultipartBody (pack "boundary")
+                      (pack "--boundary\r\ncontent-type: text/plain\r\n\r\n1\r\n--boundary--\r\nend")
+
+    , ([BodyPart (pack "content-type: text/plain\r\n") (pack "1")], Nothing) @=?
+           parseMultipartBody (pack "boundary.with.dot")
+                      (pack "--boundary.with.dot\r\ncontent-type: text/plain\r\n\r\n1\r\n--boundary.with.dot--\r\nend")
+
+    , ([BodyPart (pack "content-type: text/plain\r\n") (pack "1")], Nothing) @=?
+           parseMultipartBody (pack "boundary")
+                      (pack "beg\r\n--boundary\r\ncontent-type: text/plain\r\n\r\n1\r\n--boundary--\r\nend")
+
+    , ([BodyPart (pack "content-type: text/plain\r\n") (pack "1\n")], Nothing) @=?
+           parseMultipartBody (pack "boundary")
+                      (pack "beg\r\n--boundary\r\ncontent-type: text/plain\r\n\r\n1\n\r\n--boundary--\r\nend")
+
+    , ([BodyPart (pack "content-type: text/plain\r\n") (pack "1\r\n")], Nothing) @=?
+           parseMultipartBody (pack "boundary")
+                      (pack "beg\r\n--boundary\r\ncontent-type: text/plain\r\n\r\n1\r\n\r\n--boundary--\r\nend")
+    , ([BodyPart (pack "content-type: text/plain\r\n") (pack "1\n\r")], Nothing) @=?
+           parseMultipartBody (pack "boundary")
+                      (pack "beg\r\n--boundary\r\ncontent-type: text/plain\r\n\r\n1\n\r\r\n--boundary--\r\nend")
+
+    , ([BodyPart (pack "content-type: text/plain\r\n") (pack "\r\n1\n\r")], Nothing) @=?
+           parseMultipartBody (pack "boundary")
+                      (pack "beg\r\n--boundary\r\ncontent-type: text/plain\r\n\r\n\r\n1\n\r\r\n--boundary--\r\nend")
     ]
--}
+
 compressFilterResponseTest :: Test
 compressFilterResponseTest =
     "compressFilterResponseTest" ~: 
@@ -118,6 +132,7 @@ mkRequest method uri cookies headers body =
                         , rqHeaders     = headers
                         , rqBody        = b
                         , rqPeer        = ("",0)
+                        , rqSecure      = False
                         }
 
 compressPart :: ServerPart Response
